// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
/*jshint esnext: true */
/*jshint indent: 4 */

/* Numix/Ozon Project 2014
 *
 * Extension's version: 0.3.1
 *
 * 0.3.1 Changes:
 *  - Fixed invisible box which prevent clicking even when dock is hidden
 *  - Show Applications label is now on top of dock
 *  - Icons' popup menu is now on top of dock
 *  - Fix Y position bug when switching workspace between docks with different icon size
 *
 * 0.3 Changes:
 *  - implemented per-workspace-app behavior
 *  - reverse design direction from using :overview to use :desktop pseudo class
 *  - added option to uninstall with "make uninstall"
 *
 * 0.2.2 Changes:
 *  - add pseudo class :overview
 *  - added theme handling when gnome-shell theme changed
 *  - changed all nos-prefixes to atom
 *
 * 0.2.1 Changes:
 *  - change GnomeDash to Lang Class
 *  - fixed drag and drop behavior
 *  - icon label now appear on top of dock
 *  - more cleaning up
 *
 * 0.2 Changes:
 *  - added intellihide.js, implemented intellihide
 *
 * 0.1.1 Changes:
 *  - remove hardcoded css, add theme support
 *  - added Legacy Overview padding-bottom so its elements won't be behind dock
 *  - fixed incorrect icon sizes on some initialization
 *  - implemented 75% of screen width as maximum dock width instead of 100%
 *  - changed possible icon size range to 24-48px for esthetic reasons
 *
 * 0.1 Changes:
 *  - added NosDock to contain NosDash and handle intellihide behaviors
 *  - added NosDash containing favorite and running apps list
 *  - hide dock's background on overview, taken from nos-panel
 *  - convert indentation to spaces, added emacs header line
 *
 * TODO(s):
 *  - bug on each initialization, dock stuck not hiding until hovered out
 *  - add animation of adding-removing app icon on redisplay and only shows that
 *    animation when dock is not hidden
 *  - app label only shows when animation not running (see #18)
 *  - check behavior on multiple monitor
 *  - add settings schema
 *  - implement workspace button
 *
 */

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Gio = imports.gi.Gio;
const GnomeDash = Me.imports.gnomedash;
const Intellihide = Me.imports.intellihide;
const AtomDock = Me.imports.atomdock;

// Disable message tray
const LayoutManager = imports.ui.main.layoutManager;
const MessageTray = imports.ui.main.messageTray;
const Shell = imports.gi.Shell;
let _myDwell = 0;

let oldDash;
let atomDock;
let intellihide;
let settings = null;
let settingsSignal = null;
let settingsDisableMessageTray = true;


// Settings
const SETTINGS_DISABLE_MESSAGE_TRAY = "disable-message-tray";

// Disable message tray
function disableMessageTray() {

    _myDwell = MessageTray._trayDwellTimeout;

    if (settingsDisableMessageTray) {
        if("_trayPressure" in LayoutManager) {
            LayoutManager._trayPressure._keybindingMode =
                Shell.KeyBindingMode.OVERVIEW;
        }
        
        MessageTray._trayDwellTimeout = function() { return false; };
    }
}

// Enable message tray
function enableMessageTray() {

    if("_trayPressure" in LayoutManager) {
        LayoutManager._trayPressure._keybindingMode = 
            Shell.KeyBindingMode.NORMAL | Shell.KeyBindingMode.OVERVIEW;
    }

    if(_myDwell) {
        MessageTray._trayDwellTimeout = _myDwell;
        _myDwell = 0;
    }
}

function settingsChanged() {
    settingsDisableMessageTray = settings.get_boolean(SETTINGS_DISABLE_MESSAGE_TRAY);
    if (settingsDisableMessageTray) {
        disableMessageTray();
    } else {
        enableMessageTray();
    }
}

// Init settings
function initSettings() {
    const GioSSS = Gio.SettingsSchemaSource;
    let schemaSource = GioSSS.new_from_directory(Me.path + "/schemas", 
            GioSSS.get_default(), false);

    let schemaObj = schemaSource.lookup(Me.metadata["settings-schema"], true);
    if(!schemaObj) {
        throw new Error("Schema " + Me.metadata["settings-schema"] + " could not be found for extension " +
                        Me.uuid + ". Please check your installation.");
    }
    settings = new Gio.Settings({ settings_schema: schemaObj });
    settingsDisableMessageTray = settings.get_boolean(SETTINGS_DISABLE_MESSAGE_TRAY);
}

function init() {
    oldDash = new GnomeDash.GnomeDash();

    initSettings();
}

function show() {
    atomDock.disableAutoHide();
}

function hide() {
    atomDock.enableAutoHide();
}

function retop() {
    atomDock.retop();
}

function enable() {
    // Hide old dash
    oldDash.hideDash();

    // Disable message tray
    disableMessageTray();

    // Enable new dock
    atomDock = new AtomDock.AtomDock();
    intellihide = new Intellihide.Intellihide(show, hide, retop, atomDock);

    settingsSignal = settings.connect("changed::" + SETTINGS_DISABLE_MESSAGE_TRAY,
        function() { settingsChanged(); });
}

function disable() {
    intellihide.destroy();
    atomDock.destroy();
    oldDash.showDash();
    
    enableMessageTray();
    
    if(_myDwell) {
        MessageTray._trayDwellTimeout = _myDwell;
        _myDwell = 0;
    }

    if(settingsSignal != null) {
        settings.disconnect(settingsSignal);
        settingsSignal = null;
    }
}
